from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.get_segment_context_context_output import GetSegmentContextContextOutput
from ...models.segment_contexts import SegmentContexts
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_name: str,
    *,
    client: Client,
    document_identifier: str,
    segment_start: int,
    from_before: Union[Unset, None, int] = 0,
    size_before: Union[Unset, None, int] = 1,
    from_after: Union[Unset, None, int] = 0,
    size_after: Union[Unset, None, int] = 1,
    context_output: Union[Unset, None, GetSegmentContextContextOutput] = GetSegmentContextContextOutput.SEGMENTS,
    include_annotations: Union[Unset, None, bool] = False,
    html_version: Union[Unset, None, bool] = False,
) -> Dict[str, Any]:
    url = "{}/projects/{projectName}/segments/_context".format(client.base_url, projectName=project_name)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_context_output: Union[Unset, None, str] = UNSET
    if not isinstance(context_output, Unset):
        json_context_output = context_output.value if context_output else None

    params: Dict[str, Any] = {
        "documentIdentifier": document_identifier,
        "segmentStart": segment_start,
        "fromBefore": from_before,
        "sizeBefore": size_before,
        "fromAfter": from_after,
        "sizeAfter": size_after,
        "contextOutput": json_context_output,
        "includeAnnotations": include_annotations,
        "htmlVersion": html_version,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[SegmentContexts]:
    if response.status_code == 200:
        response_200 = SegmentContexts.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[SegmentContexts]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    project_name: str,
    *,
    client: Client,
    document_identifier: str,
    segment_start: int,
    from_before: Union[Unset, None, int] = 0,
    size_before: Union[Unset, None, int] = 1,
    from_after: Union[Unset, None, int] = 0,
    size_after: Union[Unset, None, int] = 1,
    context_output: Union[Unset, None, GetSegmentContextContextOutput] = GetSegmentContextContextOutput.SEGMENTS,
    include_annotations: Union[Unset, None, bool] = False,
    html_version: Union[Unset, None, bool] = False,
) -> Response[SegmentContexts]:
    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
        document_identifier=document_identifier,
        segment_start=segment_start,
        from_before=from_before,
        size_before=size_before,
        from_after=from_after,
        size_after=size_after,
        context_output=context_output,
        include_annotations=include_annotations,
        html_version=html_version,
    )

    response = httpx.post(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    project_name: str,
    *,
    client: Client,
    document_identifier: str,
    segment_start: int,
    from_before: Union[Unset, None, int] = 0,
    size_before: Union[Unset, None, int] = 1,
    from_after: Union[Unset, None, int] = 0,
    size_after: Union[Unset, None, int] = 1,
    context_output: Union[Unset, None, GetSegmentContextContextOutput] = GetSegmentContextContextOutput.SEGMENTS,
    include_annotations: Union[Unset, None, bool] = False,
    html_version: Union[Unset, None, bool] = False,
) -> Optional[SegmentContexts]:
    """ """

    return sync_detailed(
        project_name=project_name,
        client=client,
        document_identifier=document_identifier,
        segment_start=segment_start,
        from_before=from_before,
        size_before=size_before,
        from_after=from_after,
        size_after=size_after,
        context_output=context_output,
        include_annotations=include_annotations,
        html_version=html_version,
    ).parsed


async def asyncio_detailed(
    project_name: str,
    *,
    client: Client,
    document_identifier: str,
    segment_start: int,
    from_before: Union[Unset, None, int] = 0,
    size_before: Union[Unset, None, int] = 1,
    from_after: Union[Unset, None, int] = 0,
    size_after: Union[Unset, None, int] = 1,
    context_output: Union[Unset, None, GetSegmentContextContextOutput] = GetSegmentContextContextOutput.SEGMENTS,
    include_annotations: Union[Unset, None, bool] = False,
    html_version: Union[Unset, None, bool] = False,
) -> Response[SegmentContexts]:
    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
        document_identifier=document_identifier,
        segment_start=segment_start,
        from_before=from_before,
        size_before=size_before,
        from_after=from_after,
        size_after=size_after,
        context_output=context_output,
        include_annotations=include_annotations,
        html_version=html_version,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    project_name: str,
    *,
    client: Client,
    document_identifier: str,
    segment_start: int,
    from_before: Union[Unset, None, int] = 0,
    size_before: Union[Unset, None, int] = 1,
    from_after: Union[Unset, None, int] = 0,
    size_after: Union[Unset, None, int] = 1,
    context_output: Union[Unset, None, GetSegmentContextContextOutput] = GetSegmentContextContextOutput.SEGMENTS,
    include_annotations: Union[Unset, None, bool] = False,
    html_version: Union[Unset, None, bool] = False,
) -> Optional[SegmentContexts]:
    """ """

    return (
        await asyncio_detailed(
            project_name=project_name,
            client=client,
            document_identifier=document_identifier,
            segment_start=segment_start,
            from_before=from_before,
            size_before=size_before,
            from_after=from_after,
            size_after=size_after,
            context_output=context_output,
            include_annotations=include_annotations,
            html_version=html_version,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="BearerToken")


@attr.s(auto_attribs=True)
class BearerToken:
    """ """

    email: str
    access_token: str

    def to_dict(self) -> Dict[str, Any]:
        email = self.email
        access_token = self.access_token

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "email": email,
                "access_token": access_token,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        email = d.pop("email")

        access_token = d.pop("access_token")

        bearer_token = cls(
            email=email,
            access_token=access_token,
        )

        return bearer_token



""" Contains some shared types for properties """
from typing import BinaryIO, Generic, MutableMapping, Optional, TextIO, Tuple, TypeVar, Union

import attr
from httpx import codes, HTTPStatusError


class Unset:
    def __bool__(self) -> bool:
        return False


UNSET: Unset = Unset()

FileJsonType = Tuple[Optional[str], Union[BinaryIO, TextIO], Optional[str]]


@attr.s(auto_attribs=True)
class File:
    """Contains information for file uploads"""

    payload: Union[BinaryIO, TextIO]
    file_name: Optional[str] = None
    mime_type: Optional[str] = None

    def to_tuple(self) -> FileJsonType:
        """Return a tuple representation that httpx will accept for multipart/form-data"""
        return self.file_name, self.payload, self.mime_type


T = TypeVar("T")


@attr.s(auto_attribs=True)
class Response(Generic[T]):
    """A response from an endpoint"""

    status_code: int
    content: bytes
    headers: MutableMapping[str, str]
    parsed: Optional[T]

    @property
    def is_informational(self) -> bool:
        """
        A property which is `True` for 1xx status codes, `False` otherwise.
        """
        return codes.is_informational(self.status_code)

    @property
    def is_success(self) -> bool:
        """
        A property which is `True` for 2xx status codes, `False` otherwise.
        """
        return codes.is_success(self.status_code)

    @property
    def is_redirect(self) -> bool:
        """
        A property which is `True` for 3xx status codes, `False` otherwise.

        Note that not all responses with a 3xx status code indicate a URL redirect.

        Use `response.has_redirect_location` to determine responses with a properly
        formed URL redirection.
        """
        return codes.is_redirect(self.status_code)

    @property
    def is_client_error(self) -> bool:
        """
        A property which is `True` for 4xx status codes, `False` otherwise.
        """
        return codes.is_client_error(self.status_code)

    @property
    def is_server_error(self) -> bool:
        """
        A property which is `True` for 5xx status codes, `False` otherwise.
        """
        return codes.is_server_error(self.status_code)

    @property
    def is_error(self) -> bool:
        """
        A property which is `True` for 4xx and 5xx status codes, `False` otherwise.
        """
        return codes.is_error(self.status_code)

    @property
    def has_redirect_location(self) -> bool:
        """
        Returns True for 3xx responses with a properly formed URL redirection,
        `False` otherwise.
        """
        return (
            self.status_code
            in (
                # 301 (Cacheable redirect. Method may change to GET.)
                codes.MOVED_PERMANENTLY,
                # 302 (Uncacheable redirect. Method may change to GET.)
                codes.FOUND,
                # 303 (Client should make a GET or HEAD request.)
                codes.SEE_OTHER,
                # 307 (Equiv. 302, but retain method)
                codes.TEMPORARY_REDIRECT,
                # 308 (Equiv. 301, but retain method)
                codes.PERMANENT_REDIRECT,
            )
            and "Location" in self.headers
        )

    def raise_for_status(self) -> None:
        """
        Raise the `HTTPStatusError` if one occurred.
        """
        request = self._request
        if request is None:
            raise RuntimeError(
                "Cannot call `raise_for_status` as the request " "instance has not been set on this response."
            )

        if self.is_success:
            return

        if self.has_redirect_location:
            message = (
                "{error_type} '{0.status_code} {0.reason_phrase}' for url '{0.url}'\n"
                "Redirect location: '{0.headers[location]}'\n"
                "For more information check: https://httpstatuses.com/{0.status_code}"
            )
        else:
            message = (
                "{error_type} '{0.status_code} {0.reason_phrase}' for url '{0.url}'\n"
                "For more information check: https://httpstatuses.com/{0.status_code}"
            )

        status_class = self.status_code // 100
        error_types = {
            1: "Informational response",
            3: "Redirect response",
            4: "Client error",
            5: "Server error",
        }
        error_type = error_types.get(status_class, "Invalid status code")
        message = message.format(self, error_type=error_type)
        raise HTTPStatusError(message, request=request, response=self)


__all__ = ["File", "Response", "FileJsonType"]



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.classification_config import ClassificationConfig
from ..types import UNSET, Unset

T = TypeVar("T", bound="ProjectBean")


@attr.s(auto_attribs=True)
class ProjectBean:
    """ """

    name: str
    label: str
    description: str
    image: str
    lang: str
    version: Union[Unset, str] = UNSET
    documents: Union[Unset, int] = UNSET
    segments: Union[Unset, int] = UNSET
    annotations: Union[Unset, int] = UNSET
    categories: Union[Unset, int] = UNSET
    nature: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET
    created_date: Union[Unset, str] = UNSET
    owner: Union[Unset, str] = UNSET
    group_name: Union[Unset, str] = UNSET
    shared: Union[Unset, bool] = UNSET
    read_only: Union[Unset, bool] = UNSET
    private: Union[Unset, bool] = UNSET
    engines: Union[Unset, List[str]] = UNSET
    algorithms: Union[Unset, List[str]] = UNSET
    metafacets: Union[Unset, List[Any]] = UNSET
    classification: Union[Unset, ClassificationConfig] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        label = self.label
        description = self.description
        image = self.image
        lang = self.lang
        version = self.version
        documents = self.documents
        segments = self.segments
        annotations = self.annotations
        categories = self.categories
        nature = self.nature
        created_by = self.created_by
        created_date = self.created_date
        owner = self.owner
        group_name = self.group_name
        shared = self.shared
        read_only = self.read_only
        private = self.private
        engines: Union[Unset, List[str]] = UNSET
        if not isinstance(self.engines, Unset):
            engines = self.engines

        algorithms: Union[Unset, List[str]] = UNSET
        if not isinstance(self.algorithms, Unset):
            algorithms = self.algorithms

        metafacets: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.metafacets, Unset):
            metafacets = []
            for metafacets_item_data in self.metafacets:
                metafacets_item = metafacets_item_data

                metafacets.append(metafacets_item)

        classification: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.classification, Unset):
            classification = self.classification.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
                "label": label,
                "description": description,
                "image": image,
                "lang": lang,
            }
        )
        if version is not UNSET:
            field_dict["version"] = version
        if documents is not UNSET:
            field_dict["documents"] = documents
        if segments is not UNSET:
            field_dict["segments"] = segments
        if annotations is not UNSET:
            field_dict["annotations"] = annotations
        if categories is not UNSET:
            field_dict["categories"] = categories
        if nature is not UNSET:
            field_dict["nature"] = nature
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if created_date is not UNSET:
            field_dict["createdDate"] = created_date
        if owner is not UNSET:
            field_dict["owner"] = owner
        if group_name is not UNSET:
            field_dict["groupName"] = group_name
        if shared is not UNSET:
            field_dict["shared"] = shared
        if read_only is not UNSET:
            field_dict["readOnly"] = read_only
        if private is not UNSET:
            field_dict["private"] = private
        if engines is not UNSET:
            field_dict["engines"] = engines
        if algorithms is not UNSET:
            field_dict["algorithms"] = algorithms
        if metafacets is not UNSET:
            field_dict["metafacets"] = metafacets
        if classification is not UNSET:
            field_dict["classification"] = classification

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        label = d.pop("label")

        description = d.pop("description")

        image = d.pop("image")

        lang = d.pop("lang")

        version = d.pop("version", UNSET)

        documents = d.pop("documents", UNSET)

        segments = d.pop("segments", UNSET)

        annotations = d.pop("annotations", UNSET)

        categories = d.pop("categories", UNSET)

        nature = d.pop("nature", UNSET)

        created_by = d.pop("createdBy", UNSET)

        created_date = d.pop("createdDate", UNSET)

        owner = d.pop("owner", UNSET)

        group_name = d.pop("groupName", UNSET)

        shared = d.pop("shared", UNSET)

        read_only = d.pop("readOnly", UNSET)

        private = d.pop("private", UNSET)

        engines = cast(List[str], d.pop("engines", UNSET))

        algorithms = cast(List[str], d.pop("algorithms", UNSET))

        metafacets = []
        _metafacets = d.pop("metafacets", UNSET)
        for metafacets_item_data in _metafacets or []:
            metafacets_item = metafacets_item_data

            metafacets.append(metafacets_item)

        _classification = d.pop("classification", UNSET)
        classification: Union[Unset, ClassificationConfig]
        if isinstance(_classification, Unset):
            classification = UNSET
        else:
            classification = ClassificationConfig.from_dict(_classification)

        project_bean = cls(
            name=name,
            label=label,
            description=description,
            image=image,
            lang=lang,
            version=version,
            documents=documents,
            segments=segments,
            annotations=annotations,
            categories=categories,
            nature=nature,
            created_by=created_by,
            created_date=created_date,
            owner=owner,
            group_name=group_name,
            shared=shared,
            read_only=read_only,
            private=private,
            engines=engines,
            algorithms=algorithms,
            metafacets=metafacets,
            classification=classification,
        )

        return project_bean



from typing import Any, Dict, Type, TypeVar

import attr

from ..models.segment import Segment

T = TypeVar("T", bound="SegmentHit")


@attr.s(auto_attribs=True)
class SegmentHit:
    """ """

    score: float
    id: str
    segment: Segment

    def to_dict(self) -> Dict[str, Any]:
        score = self.score
        id = self.id
        segment = self.segment.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "score": score,
                "_id": id,
                "segment": segment,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        score = d.pop("score")

        id = d.pop("_id")

        segment = Segment.from_dict(d.pop("segment"))

        segment_hit = cls(
            score=score,
            id=id,
            segment=segment,
        )

        return segment_hit



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Annotator")


@attr.s(auto_attribs=True)
class Annotator:
    """ """

    name: str
    label: str
    engine: str
    favorite: bool
    is_default: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        label = self.label
        engine = self.engine
        favorite = self.favorite
        is_default = self.is_default

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
                "label": label,
                "engine": engine,
                "favorite": favorite,
            }
        )
        if is_default is not UNSET:
            field_dict["isDefault"] = is_default

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        label = d.pop("label")

        engine = d.pop("engine")

        favorite = d.pop("favorite")

        is_default = d.pop("isDefault", UNSET)

        annotator = cls(
            name=name,
            label=label,
            engine=engine,
            favorite=favorite,
            is_default=is_default,
        )

        return annotator



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.converter import Converter
from ..models.with_annotator import WithAnnotator
from ..models.with_processor import WithProcessor

T = TypeVar("T", bound="ConvertAnnotationPlan")


@attr.s(auto_attribs=True)
class ConvertAnnotationPlan:
    """ """

    converter: Converter
    pipeline: List[Union[WithAnnotator, WithProcessor]]

    def to_dict(self) -> Dict[str, Any]:
        converter = self.converter.to_dict()

        pipeline = []
        for pipeline_item_data in self.pipeline:
            if isinstance(pipeline_item_data, WithAnnotator):
                pipeline_item = pipeline_item_data.to_dict()

            else:
                pipeline_item = pipeline_item_data.to_dict()

            pipeline.append(pipeline_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "converter": converter,
                "pipeline": pipeline,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        converter = Converter.from_dict(d.pop("converter"))

        pipeline = []
        _pipeline = d.pop("pipeline")
        for pipeline_item_data in _pipeline:

            def _parse_pipeline_item(data: object) -> Union[WithAnnotator, WithProcessor]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    pipeline_item_type_0 = WithAnnotator.from_dict(data)

                    return pipeline_item_type_0
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                pipeline_item_type_1 = WithProcessor.from_dict(data)

                return pipeline_item_type_1

            pipeline_item = _parse_pipeline_item(pipeline_item_data)

            pipeline.append(pipeline_item)

        convert_annotation_plan = cls(
            converter=converter,
            pipeline=pipeline,
        )

        return convert_annotation_plan



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.annotated_document import AnnotatedDocument
from ...models.input_document import InputDocument
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_name: str,
    annotator: str,
    *,
    client: Client,
    json_body: List[InputDocument],
    inline_labels: Union[Unset, None, bool] = True,
    inline_label_ids: Union[Unset, None, bool] = True,
    inline_text: Union[Unset, None, bool] = True,
    output_fields: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/projects/{projectName}/annotators/{annotator}/_annotate_documents".format(
        client.base_url, projectName=project_name, annotator=annotator
    )

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {
        "inlineLabels": inline_labels,
        "inlineLabelIds": inline_label_ids,
        "inlineText": inline_text,
        "outputFields": output_fields,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = []
    for componentsschemas_input_document_array_item_data in json_body:
        componentsschemas_input_document_array_item = componentsschemas_input_document_array_item_data.to_dict()

        json_json_body.append(componentsschemas_input_document_array_item)

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[List[AnnotatedDocument]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for componentsschemas_annotated_document_array_item_data in _response_200:
            componentsschemas_annotated_document_array_item = AnnotatedDocument.from_dict(
                componentsschemas_annotated_document_array_item_data
            )

            response_200.append(componentsschemas_annotated_document_array_item)

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[List[AnnotatedDocument]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    project_name: str,
    annotator: str,
    *,
    client: Client,
    json_body: List[InputDocument],
    inline_labels: Union[Unset, None, bool] = True,
    inline_label_ids: Union[Unset, None, bool] = True,
    inline_text: Union[Unset, None, bool] = True,
    output_fields: Union[Unset, None, str] = UNSET,
) -> Response[List[AnnotatedDocument]]:
    kwargs = _get_kwargs(
        project_name=project_name,
        annotator=annotator,
        client=client,
        json_body=json_body,
        inline_labels=inline_labels,
        inline_label_ids=inline_label_ids,
        inline_text=inline_text,
        output_fields=output_fields,
    )

    response = httpx.post(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    project_name: str,
    annotator: str,
    *,
    client: Client,
    json_body: List[InputDocument],
    inline_labels: Union[Unset, None, bool] = True,
    inline_label_ids: Union[Unset, None, bool] = True,
    inline_text: Union[Unset, None, bool] = True,
    output_fields: Union[Unset, None, str] = UNSET,
) -> Optional[List[AnnotatedDocument]]:
    """ """

    return sync_detailed(
        project_name=project_name,
        annotator=annotator,
        client=client,
        json_body=json_body,
        inline_labels=inline_labels,
        inline_label_ids=inline_label_ids,
        inline_text=inline_text,
        output_fields=output_fields,
    ).parsed


async def asyncio_detailed(
    project_name: str,
    annotator: str,
    *,
    client: Client,
    json_body: List[InputDocument],
    inline_labels: Union[Unset, None, bool] = True,
    inline_label_ids: Union[Unset, None, bool] = True,
    inline_text: Union[Unset, None, bool] = True,
    output_fields: Union[Unset, None, str] = UNSET,
) -> Response[List[AnnotatedDocument]]:
    kwargs = _get_kwargs(
        project_name=project_name,
        annotator=annotator,
        client=client,
        json_body=json_body,
        inline_labels=inline_labels,
        inline_label_ids=inline_label_ids,
        inline_text=inline_text,
        output_fields=output_fields,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    project_name: str,
    annotator: str,
    *,
    client: Client,
    json_body: List[InputDocument],
    inline_labels: Union[Unset, None, bool] = True,
    inline_label_ids: Union[Unset, None, bool] = True,
    inline_text: Union[Unset, None, bool] = True,
    output_fields: Union[Unset, None, str] = UNSET,
) -> Optional[List[AnnotatedDocument]]:
    """ """

    return (
        await asyncio_detailed(
            project_name=project_name,
            annotator=annotator,
            client=client,
            json_body=json_body,
            inline_labels=inline_labels,
            inline_label_ids=inline_label_ids,
            inline_text=inline_text,
            output_fields=output_fields,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.with_annotator_condition import WithAnnotatorCondition
from ..models.with_annotator_parameters import WithAnnotatorParameters
from ..types import UNSET, Unset

T = TypeVar("T", bound="WithAnnotator")


@attr.s(auto_attribs=True)
class WithAnnotator:
    """ """

    annotator: str
    disabled: Union[Unset, bool] = UNSET
    project_name: Union[Unset, str] = UNSET
    parameters: Union[Unset, WithAnnotatorParameters] = UNSET
    condition: Union[Unset, WithAnnotatorCondition] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        annotator = self.annotator
        disabled = self.disabled
        project_name = self.project_name
        parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parameters, Unset):
            parameters = self.parameters.to_dict()

        condition: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.condition, Unset):
            condition = self.condition.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "annotator": annotator,
            }
        )
        if disabled is not UNSET:
            field_dict["disabled"] = disabled
        if project_name is not UNSET:
            field_dict["projectName"] = project_name
        if parameters is not UNSET:
            field_dict["parameters"] = parameters
        if condition is not UNSET:
            field_dict["condition"] = condition

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        annotator = d.pop("annotator")

        disabled = d.pop("disabled", UNSET)

        project_name = d.pop("projectName", UNSET)

        _parameters = d.pop("parameters", UNSET)
        parameters: Union[Unset, WithAnnotatorParameters]
        if isinstance(_parameters, Unset):
            parameters = UNSET
        else:
            parameters = WithAnnotatorParameters.from_dict(_parameters)

        _condition = d.pop("condition", UNSET)
        condition: Union[Unset, WithAnnotatorCondition]
        if isinstance(_condition, Unset):
            condition = UNSET
        else:
            condition = WithAnnotatorCondition.from_dict(_condition)

        with_annotator = cls(
            annotator=annotator,
            disabled=disabled,
            project_name=project_name,
            parameters=parameters,
            condition=condition,
        )

        return with_annotator



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Label")


@attr.s(auto_attribs=True)
class Label:
    """ """

    name: str
    label: str
    color: str
    identifier: Union[Unset, str] = UNSET
    count: Union[Unset, int] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        label = self.label
        color = self.color
        identifier = self.identifier
        count = self.count

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
                "label": label,
                "color": color,
            }
        )
        if identifier is not UNSET:
            field_dict["identifier"] = identifier
        if count is not UNSET:
            field_dict["count"] = count

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        label = d.pop("label")

        color = d.pop("color")

        identifier = d.pop("identifier", UNSET)

        count = d.pop("count", UNSET)

        label = cls(
            name=name,
            label=label,
            color=color,
            identifier=identifier,
            count=count,
        )

        return label



from typing import Any, Dict, List, Optional

import httpx

from ...client import Client
from ...models.engine_config_import_summary import EngineConfigImportSummary
from ...models.uploaded_file import UploadedFile
from ...types import Response


def _get_kwargs(
    project_name: str,
    *,
    client: Client,
    json_body: List[UploadedFile],
) -> Dict[str, Any]:
    url = "{}/projects/{projectName}/_load_models".format(client.base_url, projectName=project_name)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = []
    for componentsschemas_uploaded_file_array_item_data in json_body:
        componentsschemas_uploaded_file_array_item = componentsschemas_uploaded_file_array_item_data.to_dict()

        json_json_body.append(componentsschemas_uploaded_file_array_item)

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> Optional[EngineConfigImportSummary]:
    if response.status_code == 200:
        response_200 = EngineConfigImportSummary.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[EngineConfigImportSummary]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    project_name: str,
    *,
    client: Client,
    json_body: List[UploadedFile],
) -> Response[EngineConfigImportSummary]:
    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
        json_body=json_body,
    )

    response = httpx.post(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    project_name: str,
    *,
    client: Client,
    json_body: List[UploadedFile],
) -> Optional[EngineConfigImportSummary]:
    """ """

    return sync_detailed(
        project_name=project_name,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    project_name: str,
    *,
    client: Client,
    json_body: List[UploadedFile],
) -> Response[EngineConfigImportSummary]:
    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    project_name: str,
    *,
    client: Client,
    json_body: List[UploadedFile],
) -> Optional[EngineConfigImportSummary]:
    """ """

    return (
        await asyncio_detailed(
            project_name=project_name,
            client=client,
            json_body=json_body,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.label_count import LabelCount
from ..models.text_count import TextCount

T = TypeVar("T", bound="SuggestionFacets")


@attr.s(auto_attribs=True)
class SuggestionFacets:
    """ """

    labels: List[LabelCount]
    texts: List[TextCount]

    def to_dict(self) -> Dict[str, Any]:
        labels = []
        for labels_item_data in self.labels:
            labels_item = labels_item_data.to_dict()

            labels.append(labels_item)

        texts = []
        for texts_item_data in self.texts:
            texts_item = texts_item_data.to_dict()

            texts.append(texts_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "labels": labels,
                "texts": texts,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        labels = []
        _labels = d.pop("labels")
        for labels_item_data in _labels:
            labels_item = LabelCount.from_dict(labels_item_data)

            labels.append(labels_item)

        texts = []
        _texts = d.pop("texts")
        for texts_item_data in _texts:
            texts_item = TextCount.from_dict(texts_item_data)

            texts.append(texts_item)

        suggestion_facets = cls(
            labels=labels,
            texts=texts,
        )

        return suggestion_facets



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="ClassificationConfig")


@attr.s(auto_attribs=True)
class ClassificationConfig:
    """ """

    exclusive_classes: bool

    def to_dict(self) -> Dict[str, Any]:
        exclusive_classes = self.exclusive_classes

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "exclusive_classes": exclusive_classes,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        exclusive_classes = d.pop("exclusive_classes")

        classification_config = cls(
            exclusive_classes=exclusive_classes,
        )

        return classification_config



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.with_processor_condition import WithProcessorCondition
from ..models.with_processor_parameters import WithProcessorParameters
from ..types import UNSET, Unset

T = TypeVar("T", bound="WithProcessor")


@attr.s(auto_attribs=True)
class WithProcessor:
    """ """

    processor: str
    disabled: Union[Unset, bool] = UNSET
    parameters: Union[Unset, WithProcessorParameters] = UNSET
    condition: Union[Unset, WithProcessorCondition] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        processor = self.processor
        disabled = self.disabled
        parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parameters, Unset):
            parameters = self.parameters.to_dict()

        condition: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.condition, Unset):
            condition = self.condition.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "processor": processor,
            }
        )
        if disabled is not UNSET:
            field_dict["disabled"] = disabled
        if parameters is not UNSET:
            field_dict["parameters"] = parameters
        if condition is not UNSET:
            field_dict["condition"] = condition

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        processor = d.pop("processor")

        disabled = d.pop("disabled", UNSET)

        _parameters = d.pop("parameters", UNSET)
        parameters: Union[Unset, WithProcessorParameters]
        if isinstance(_parameters, Unset):
            parameters = UNSET
        else:
            parameters = WithProcessorParameters.from_dict(_parameters)

        _condition = d.pop("condition", UNSET)
        condition: Union[Unset, WithProcessorCondition]
        if isinstance(_condition, Unset):
            condition = UNSET
        else:
            condition = WithProcessorCondition.from_dict(_condition)

        with_processor = cls(
            processor=processor,
            disabled=disabled,
            parameters=parameters,
            condition=condition,
        )

        return with_processor



import ssl
from typing import Dict, Union, Optional

import attr

from sherpa_client.models import Credentials, RequestJwtTokenProjectAccessMode, BearerToken
from sherpa_client.types import Unset, UNSET, Response


@attr.s(auto_attribs=True)
class Client:
    """A class for keeping track of data related to the API"""

    base_url: str
    cookies: Dict[str, str] = attr.ib(factory=dict, kw_only=True)
    headers: Dict[str, str] = attr.ib(factory=dict, kw_only=True)
    timeout: float = attr.ib(5.0, kw_only=True)
    verify_ssl: Union[str, bool, ssl.SSLContext] = attr.ib(True, kw_only=True)

    def get_headers(self) -> Dict[str, str]:
        """Get headers to be used in all endpoints"""
        return {**self.headers}

    def with_headers(self, headers: Dict[str, str]) -> "Client":
        """Get a new client matching this one with additional headers"""
        return attr.evolve(self, headers={**self.headers, **headers})

    def get_cookies(self) -> Dict[str, str]:
        return {**self.cookies}

    def with_cookies(self, cookies: Dict[str, str]) -> "Client":
        """Get a new client matching this one with additional cookies"""
        return attr.evolve(self, cookies={**self.cookies, **cookies})

    def get_timeout(self) -> float:
        return self.timeout

    def with_timeout(self, timeout: float) -> "Client":
        """Get a new client matching this one with a new timeout (in seconds)"""
        return attr.evolve(self, timeout=timeout)


@attr.s(auto_attribs=True)
class AuthenticatedClient(Client):
    """A Client which has been authenticated for use on secured endpoints"""

    token: str = attr.ib(init=False, default=None)

    def get_headers(self) -> Dict[str, str]:
        """Get headers to be used in authenticated endpoints"""
        return {"Authorization": f"Bearer {self.token}", **self.headers}

    def login(
        self,
        json_body: Credentials,
        project_filter: Union[Unset, None, str] = UNSET,
        project_access_mode: Union[Unset, None, RequestJwtTokenProjectAccessMode] = UNSET,
        annotate_only: Union[Unset, None, bool] = False,
        login_only: Union[Unset, None, bool] = False,
    ):
        """ """
        from sherpa_client.api.authentication import request_jwt_token
        r: Response[BearerToken] = request_jwt_token.sync_detailed(
            self,
            json_body=json_body,
            project_filter=project_filter,
            project_access_mode=project_access_mode,
            annotate_only=annotate_only,
            login_only=login_only,
        )
        if r.is_success:
            self.token = r.parsed.access_token
        else:
            r.raise_for_status()



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="UserProfileUpdate")


@attr.s(auto_attribs=True)
class UserProfileUpdate:
    """ """

    profilename: Union[Unset, str] = UNSET
    password: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        profilename = self.profilename
        password = self.password

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if profilename is not UNSET:
            field_dict["profilename"] = profilename
        if password is not UNSET:
            field_dict["password"] = password

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        profilename = d.pop("profilename", UNSET)

        password = d.pop("password", UNSET)

        user_profile_update = cls(
            profilename=profilename,
            password=password,
        )

        return user_profile_update



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="ShareMode")


@attr.s(auto_attribs=True)
class ShareMode:
    """ """

    read: bool
    write: bool

    def to_dict(self) -> Dict[str, Any]:
        read = self.read
        write = self.write

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "read": read,
                "write": write,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        read = d.pop("read")

        write = d.pop("write")

        share_mode = cls(
            read=read,
            write=write,
        )

        return share_mode



from typing import Any, Dict, List, Type, TypeVar, cast

import attr

T = TypeVar("T", bound="NewRole")


@attr.s(auto_attribs=True)
class NewRole:
    """ """

    label: str
    permissions: List[str]

    def to_dict(self) -> Dict[str, Any]:
        label = self.label
        permissions = self.permissions

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "label": label,
                "permissions": permissions,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        label = d.pop("label")

        permissions = cast(List[str], d.pop("permissions"))

        new_role = cls(
            label=label,
            permissions=permissions,
        )

        return new_role



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="SimpleMetadata")


@attr.s(auto_attribs=True)
class SimpleMetadata:
    """ """

    name: str
    value: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        value = self.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
            }
        )
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        value = d.pop("value", UNSET)

        simple_metadata = cls(
            name=name,
            value=value,
        )

        return simple_metadata



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.classification_options import ClassificationOptions
from ..types import UNSET, Unset

T = TypeVar("T", bound="ConfigPatchOptions")


@attr.s(auto_attribs=True)
class ConfigPatchOptions:
    """ """

    classification: Union[Unset, ClassificationOptions] = UNSET
    label: Union[Unset, str] = UNSET
    metafacets: Union[Unset, List[str]] = UNSET
    image: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        classification: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.classification, Unset):
            classification = self.classification.to_dict()

        label = self.label
        metafacets: Union[Unset, List[str]] = UNSET
        if not isinstance(self.metafacets, Unset):
            metafacets = self.metafacets

        image = self.image

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if classification is not UNSET:
            field_dict["classification"] = classification
        if label is not UNSET:
            field_dict["label"] = label
        if metafacets is not UNSET:
            field_dict["metafacets"] = metafacets
        if image is not UNSET:
            field_dict["image"] = image

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _classification = d.pop("classification", UNSET)
        classification: Union[Unset, ClassificationOptions]
        if isinstance(_classification, Unset):
            classification = UNSET
        else:
            classification = ClassificationOptions.from_dict(_classification)

        label = d.pop("label", UNSET)

        metafacets = cast(List[str], d.pop("metafacets", UNSET))

        image = d.pop("image", UNSET)

        config_patch_options = cls(
            classification=classification,
            label=label,
            metafacets=metafacets,
            image=image,
        )

        return config_patch_options



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.gazetteer_patch_parameters import GazetteerPatchParameters
from ..types import UNSET, Unset

T = TypeVar("T", bound="GazetteerPatch")


@attr.s(auto_attribs=True)
class GazetteerPatch:
    """ """

    label: Union[Unset, str] = UNSET
    parameters: Union[Unset, GazetteerPatchParameters] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        label = self.label
        parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parameters, Unset):
            parameters = self.parameters.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if label is not UNSET:
            field_dict["label"] = label
        if parameters is not UNSET:
            field_dict["parameters"] = parameters

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        label = d.pop("label", UNSET)

        _parameters = d.pop("parameters", UNSET)
        parameters: Union[Unset, GazetteerPatchParameters]
        if isinstance(_parameters, Unset):
            parameters = UNSET
        else:
            parameters = GazetteerPatchParameters.from_dict(_parameters)

        gazetteer_patch = cls(
            label=label,
            parameters=parameters,
        )

        return gazetteer_patch



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="Ack")


@attr.s(auto_attribs=True)
class Ack:
    """ """

    ok: bool

    def to_dict(self) -> Dict[str, Any]:
        ok = self.ok

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "ok": ok,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        ok = d.pop("ok")

        ack = cls(
            ok=ok,
        )

        return ack



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.annotated_doc_annotation_properties import AnnotatedDocAnnotationProperties
from ..models.annotation_term import AnnotationTerm
from ..types import UNSET, Unset

T = TypeVar("T", bound="AnnotatedDocAnnotation")


@attr.s(auto_attribs=True)
class AnnotatedDocAnnotation:
    """A document annotation"""

    label_name: str
    start: int
    end: int
    text: str
    label_id: Union[Unset, str] = UNSET
    label: Union[Unset, str] = UNSET
    score: Union[Unset, float] = UNSET
    properties: Union[Unset, AnnotatedDocAnnotationProperties] = UNSET
    terms: Union[Unset, List[AnnotationTerm]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        label_name = self.label_name
        start = self.start
        end = self.end
        text = self.text
        label_id = self.label_id
        label = self.label
        score = self.score
        properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.properties, Unset):
            properties = self.properties.to_dict()

        terms: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.terms, Unset):
            terms = []
            for terms_item_data in self.terms:
                terms_item = terms_item_data.to_dict()

                terms.append(terms_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "labelName": label_name,
                "start": start,
                "end": end,
                "text": text,
            }
        )
        if label_id is not UNSET:
            field_dict["labelId"] = label_id
        if label is not UNSET:
            field_dict["label"] = label
        if score is not UNSET:
            field_dict["score"] = score
        if properties is not UNSET:
            field_dict["properties"] = properties
        if terms is not UNSET:
            field_dict["terms"] = terms

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        label_name = d.pop("labelName")

        start = d.pop("start")

        end = d.pop("end")

        text = d.pop("text")

        label_id = d.pop("labelId", UNSET)

        label = d.pop("label", UNSET)

        score = d.pop("score", UNSET)

        _properties = d.pop("properties", UNSET)
        properties: Union[Unset, AnnotatedDocAnnotationProperties]
        if isinstance(_properties, Unset):
            properties = UNSET
        else:
            properties = AnnotatedDocAnnotationProperties.from_dict(_properties)

        terms = []
        _terms = d.pop("terms", UNSET)
        for terms_item_data in _terms or []:
            terms_item = AnnotationTerm.from_dict(terms_item_data)

            terms.append(terms_item)

        annotated_doc_annotation = cls(
            label_name=label_name,
            start=start,
            end=end,
            text=text,
            label_id=label_id,
            label=label,
            score=score,
            properties=properties,
            terms=terms,
        )

        return annotated_doc_annotation



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.term_hits import TermHits
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_name: str,
    *,
    client: Client,
    query: Union[Unset, None, str] = "",
    from_: Union[Unset, None, int] = 0,
    size: Union[Unset, None, int] = 10,
    highlight: Union[Unset, None, bool] = False,
    facet: Union[Unset, None, bool] = False,
    query_filter: Union[Unset, None, str] = "",
    output_fields: Union[Unset, None, str] = "",
    simple_query: Union[Unset, None, bool] = False,
    selected_facets: Union[Unset, None, List[str]] = UNSET,
) -> Dict[str, Any]:
    url = "{}/projects/{projectName}/lexicons/_search".format(client.base_url, projectName=project_name)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_selected_facets: Union[Unset, None, List[str]] = UNSET
    if not isinstance(selected_facets, Unset):
        if selected_facets is None:
            json_selected_facets = None
        else:
            json_selected_facets = selected_facets

    params: Dict[str, Any] = {
        "query": query,
        "from": from_,
        "size": size,
        "highlight": highlight,
        "facet": facet,
        "queryFilter": query_filter,
        "outputFields": output_fields,
        "simpleQuery": simple_query,
        "selectedFacets": json_selected_facets,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[TermHits]:
    if response.status_code == 200:
        response_200 = TermHits.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[TermHits]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    project_name: str,
    *,
    client: Client,
    query: Union[Unset, None, str] = "",
    from_: Union[Unset, None, int] = 0,
    size: Union[Unset, None, int] = 10,
    highlight: Union[Unset, None, bool] = False,
    facet: Union[Unset, None, bool] = False,
    query_filter: Union[Unset, None, str] = "",
    output_fields: Union[Unset, None, str] = "",
    simple_query: Union[Unset, None, bool] = False,
    selected_facets: Union[Unset, None, List[str]] = UNSET,
) -> Response[TermHits]:
    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
        query=query,
        from_=from_,
        size=size,
        highlight=highlight,
        facet=facet,
        query_filter=query_filter,
        output_fields=output_fields,
        simple_query=simple_query,
        selected_facets=selected_facets,
    )

    response = httpx.post(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    project_name: str,
    *,
    client: Client,
    query: Union[Unset, None, str] = "",
    from_: Union[Unset, None, int] = 0,
    size: Union[Unset, None, int] = 10,
    highlight: Union[Unset, None, bool] = False,
    facet: Union[Unset, None, bool] = False,
    query_filter: Union[Unset, None, str] = "",
    output_fields: Union[Unset, None, str] = "",
    simple_query: Union[Unset, None, bool] = False,
    selected_facets: Union[Unset, None, List[str]] = UNSET,
) -> Optional[TermHits]:
    """ """

    return sync_detailed(
        project_name=project_name,
        client=client,
        query=query,
        from_=from_,
        size=size,
        highlight=highlight,
        facet=facet,
        query_filter=query_filter,
        output_fields=output_fields,
        simple_query=simple_query,
        selected_facets=selected_facets,
    ).parsed


async def asyncio_detailed(
    project_name: str,
    *,
    client: Client,
    query: Union[Unset, None, str] = "",
    from_: Union[Unset, None, int] = 0,
    size: Union[Unset, None, int] = 10,
    highlight: Union[Unset, None, bool] = False,
    facet: Union[Unset, None, bool] = False,
    query_filter: Union[Unset, None, str] = "",
    output_fields: Union[Unset, None, str] = "",
    simple_query: Union[Unset, None, bool] = False,
    selected_facets: Union[Unset, None, List[str]] = UNSET,
) -> Response[TermHits]:
    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
        query=query,
        from_=from_,
        size=size,
        highlight=highlight,
        facet=facet,
        query_filter=query_filter,
        output_fields=output_fields,
        simple_query=simple_query,
        selected_facets=selected_facets,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    project_name: str,
    *,
    client: Client,
    query: Union[Unset, None, str] = "",
    from_: Union[Unset, None, int] = 0,
    size: Union[Unset, None, int] = 10,
    highlight: Union[Unset, None, bool] = False,
    facet: Union[Unset, None, bool] = False,
    query_filter: Union[Unset, None, str] = "",
    output_fields: Union[Unset, None, str] = "",
    simple_query: Union[Unset, None, bool] = False,
    selected_facets: Union[Unset, None, List[str]] = UNSET,
) -> Optional[TermHits]:
    """ """

    return (
        await asyncio_detailed(
            project_name=project_name,
            client=client,
            query=query,
            from_=from_,
            size=size,
            highlight=highlight,
            facet=facet,
            query_filter=query_filter,
            output_fields=output_fields,
            simple_query=simple_query,
            selected_facets=selected_facets,
        )
    ).parsed



from typing import Any, Dict, Optional

import httpx

from ...client import Client
from ...models.delete_response import DeleteResponse
from ...types import UNSET, Response


def _get_kwargs(
    project_name: str,
    *,
    client: Client,
    names: str,
) -> Dict[str, Any]:
    url = "{}/projects/{projectName}/suggesters".format(client.base_url, projectName=project_name)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {
        "names": names,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[DeleteResponse]:
    if response.status_code == 200:
        response_200 = DeleteResponse.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[DeleteResponse]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    project_name: str,
    *,
    client: Client,
    names: str,
) -> Response[DeleteResponse]:
    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
        names=names,
    )

    response = httpx.delete(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    project_name: str,
    *,
    client: Client,
    names: str,
) -> Optional[DeleteResponse]:
    """ """

    return sync_detailed(
        project_name=project_name,
        client=client,
        names=names,
    ).parsed


async def asyncio_detailed(
    project_name: str,
    *,
    client: Client,
    names: str,
) -> Response[DeleteResponse]:
    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
        names=names,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.delete(**kwargs)

    return _build_response(response=response)


async def asyncio(
    project_name: str,
    *,
    client: Client,
    names: str,
) -> Optional[DeleteResponse]:
    """ """

    return (
        await asyncio_detailed(
            project_name=project_name,
            client=client,
            names=names,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.aggregation import Aggregation
from ..models.search_total import SearchTotal
from ..models.segment_hit import SegmentHit
from ..types import UNSET, Unset

T = TypeVar("T", bound="SegmentHits")


@attr.s(auto_attribs=True)
class SegmentHits:
    """ """

    total: SearchTotal
    hits: List[SegmentHit]
    max_score: Union[Unset, float] = UNSET
    aggregations: Union[Unset, List[Aggregation]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        total = self.total.to_dict()

        hits = []
        for hits_item_data in self.hits:
            hits_item = hits_item_data.to_dict()

            hits.append(hits_item)

        max_score = self.max_score
        aggregations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.aggregations, Unset):
            aggregations = []
            for aggregations_item_data in self.aggregations:
                aggregations_item = aggregations_item_data.to_dict()

                aggregations.append(aggregations_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "total": total,
                "hits": hits,
            }
        )
        if max_score is not UNSET:
            field_dict["max_score"] = max_score
        if aggregations is not UNSET:
            field_dict["aggregations"] = aggregations

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        total = SearchTotal.from_dict(d.pop("total"))

        hits = []
        _hits = d.pop("hits")
        for hits_item_data in _hits:
            hits_item = SegmentHit.from_dict(hits_item_data)

            hits.append(hits_item)

        max_score = d.pop("max_score", UNSET)

        aggregations = []
        _aggregations = d.pop("aggregations", UNSET)
        for aggregations_item_data in _aggregations or []:
            aggregations_item = Aggregation.from_dict(aggregations_item_data)

            aggregations.append(aggregations_item)

        segment_hits = cls(
            total=total,
            hits=hits,
            max_score=max_score,
            aggregations=aggregations,
        )

        return segment_hits



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.doc_category_status import DocCategoryStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="DocCategory")


@attr.s(auto_attribs=True)
class DocCategory:
    """A document category"""

    identifier: str
    label_name: str
    score: Union[Unset, float] = UNSET
    status: Union[Unset, DocCategoryStatus] = UNSET
    created_date: Union[Unset, str] = UNSET
    modified_date: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        identifier = self.identifier
        label_name = self.label_name
        score = self.score
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        created_date = self.created_date
        modified_date = self.modified_date
        created_by = self.created_by

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "identifier": identifier,
                "labelName": label_name,
            }
        )
        if score is not UNSET:
            field_dict["score"] = score
        if status is not UNSET:
            field_dict["status"] = status
        if created_date is not UNSET:
            field_dict["createdDate"] = created_date
        if modified_date is not UNSET:
            field_dict["modifiedDate"] = modified_date
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        identifier = d.pop("identifier")

        label_name = d.pop("labelName")

        score = d.pop("score", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, DocCategoryStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = DocCategoryStatus(_status)

        created_date = d.pop("createdDate", UNSET)

        modified_date = d.pop("modifiedDate", UNSET)

        created_by = d.pop("createdBy", UNSET)

        doc_category = cls(
            identifier=identifier,
            label_name=label_name,
            score=score,
            status=status,
            created_date=created_date,
            modified_date=modified_date,
            created_by=created_by,
        )

        return doc_category



from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ...client import Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    field: str,
    name: Union[Unset, None, str] = "",
    api: Union[Unset, None, str] = "",
    engine: Union[Unset, None, str] = "",
    function: Union[Unset, None, str] = "",
    language: Union[Unset, None, str] = "",
    type: Union[Unset, None, str] = "",
    nature: Union[Unset, None, str] = "",
    version: Union[Unset, None, str] = "",
    term_importer: Union[Unset, None, str] = "",
    annotator: Union[Unset, None, str] = "",
    processor: Union[Unset, None, str] = "",
    formatter: Union[Unset, None, str] = "",
    converter: Union[Unset, None, str] = "",
) -> Dict[str, Any]:
    url = "{}/service_values".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {
        "field": field,
        "name": name,
        "api": api,
        "engine": engine,
        "function": function,
        "language": language,
        "type": type,
        "nature": nature,
        "version": version,
        "termImporter": term_importer,
        "annotator": annotator,
        "processor": processor,
        "formatter": formatter,
        "converter": converter,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[List[str]]:
    if response.status_code == 200:
        response_200 = cast(List[str], response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[List[str]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    field: str,
    name: Union[Unset, None, str] = "",
    api: Union[Unset, None, str] = "",
    engine: Union[Unset, None, str] = "",
    function: Union[Unset, None, str] = "",
    language: Union[Unset, None, str] = "",
    type: Union[Unset, None, str] = "",
    nature: Union[Unset, None, str] = "",
    version: Union[Unset, None, str] = "",
    term_importer: Union[Unset, None, str] = "",
    annotator: Union[Unset, None, str] = "",
    processor: Union[Unset, None, str] = "",
    formatter: Union[Unset, None, str] = "",
    converter: Union[Unset, None, str] = "",
) -> Response[List[str]]:
    kwargs = _get_kwargs(
        client=client,
        field=field,
        name=name,
        api=api,
        engine=engine,
        function=function,
        language=language,
        type=type,
        nature=nature,
        version=version,
        term_importer=term_importer,
        annotator=annotator,
        processor=processor,
        formatter=formatter,
        converter=converter,
    )

    response = httpx.get(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    field: str,
    name: Union[Unset, None, str] = "",
    api: Union[Unset, None, str] = "",
    engine: Union[Unset, None, str] = "",
    function: Union[Unset, None, str] = "",
    language: Union[Unset, None, str] = "",
    type: Union[Unset, None, str] = "",
    nature: Union[Unset, None, str] = "",
    version: Union[Unset, None, str] = "",
    term_importer: Union[Unset, None, str] = "",
    annotator: Union[Unset, None, str] = "",
    processor: Union[Unset, None, str] = "",
    formatter: Union[Unset, None, str] = "",
    converter: Union[Unset, None, str] = "",
) -> Optional[List[str]]:
    """ """

    return sync_detailed(
        client=client,
        field=field,
        name=name,
        api=api,
        engine=engine,
        function=function,
        language=language,
        type=type,
        nature=nature,
        version=version,
        term_importer=term_importer,
        annotator=annotator,
        processor=processor,
        formatter=formatter,
        converter=converter,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    field: str,
    name: Union[Unset, None, str] = "",
    api: Union[Unset, None, str] = "",
    engine: Union[Unset, None, str] = "",
    function: Union[Unset, None, str] = "",
    language: Union[Unset, None, str] = "",
    type: Union[Unset, None, str] = "",
    nature: Union[Unset, None, str] = "",
    version: Union[Unset, None, str] = "",
    term_importer: Union[Unset, None, str] = "",
    annotator: Union[Unset, None, str] = "",
    processor: Union[Unset, None, str] = "",
    formatter: Union[Unset, None, str] = "",
    converter: Union[Unset, None, str] = "",
) -> Response[List[str]]:
    kwargs = _get_kwargs(
        client=client,
        field=field,
        name=name,
        api=api,
        engine=engine,
        function=function,
        language=language,
        type=type,
        nature=nature,
        version=version,
        term_importer=term_importer,
        annotator=annotator,
        processor=processor,
        formatter=formatter,
        converter=converter,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    field: str,
    name: Union[Unset, None, str] = "",
    api: Union[Unset, None, str] = "",
    engine: Union[Unset, None, str] = "",
    function: Union[Unset, None, str] = "",
    language: Union[Unset, None, str] = "",
    type: Union[Unset, None, str] = "",
    nature: Union[Unset, None, str] = "",
    version: Union[Unset, None, str] = "",
    term_importer: Union[Unset, None, str] = "",
    annotator: Union[Unset, None, str] = "",
    processor: Union[Unset, None, str] = "",
    formatter: Union[Unset, None, str] = "",
    converter: Union[Unset, None, str] = "",
) -> Optional[List[str]]:
    """ """

    return (
        await asyncio_detailed(
            client=client,
            field=field,
            name=name,
            api=api,
            engine=engine,
            function=function,
            language=language,
            type=type,
            nature=nature,
            version=version,
            term_importer=term_importer,
            annotator=annotator,
            processor=processor,
            formatter=formatter,
            converter=converter,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar

import attr

from ..models.share_mode import ShareMode

T = TypeVar("T", bound="GroupShare")


@attr.s(auto_attribs=True)
class GroupShare:
    """ """

    group_name: str
    mode: ShareMode
    can_revoke: bool

    def to_dict(self) -> Dict[str, Any]:
        group_name = self.group_name
        mode = self.mode.to_dict()

        can_revoke = self.can_revoke

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "groupName": group_name,
                "mode": mode,
                "canRevoke": can_revoke,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        group_name = d.pop("groupName")

        mode = ShareMode.from_dict(d.pop("mode"))

        can_revoke = d.pop("canRevoke")

        group_share = cls(
            group_name=group_name,
            mode=mode,
            can_revoke=can_revoke,
        )

        return group_share



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.doc_annotation import DocAnnotation
from ..models.doc_category import DocCategory
from ..models.doc_sentence import DocSentence
from ..models.document_metadata import DocumentMetadata
from ..types import UNSET, Unset

T = TypeVar("T", bound="Document")


@attr.s(auto_attribs=True)
class Document:
    """ """

    identifier: str
    title: str
    text: str
    metadata: Union[Unset, DocumentMetadata] = UNSET
    sentences: Union[Unset, List[DocSentence]] = UNSET
    categories: Union[Unset, List[DocCategory]] = UNSET
    annotations: Union[Unset, List[DocAnnotation]] = UNSET
    created_date: Union[Unset, str] = UNSET
    modified_date: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        identifier = self.identifier
        title = self.title
        text = self.text
        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        sentences: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.sentences, Unset):
            sentences = []
            for sentences_item_data in self.sentences:
                sentences_item = sentences_item_data.to_dict()

                sentences.append(sentences_item)

        categories: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.categories, Unset):
            categories = []
            for categories_item_data in self.categories:
                categories_item = categories_item_data.to_dict()

                categories.append(categories_item)

        annotations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.annotations, Unset):
            annotations = []
            for annotations_item_data in self.annotations:
                annotations_item = annotations_item_data.to_dict()

                annotations.append(annotations_item)

        created_date = self.created_date
        modified_date = self.modified_date
        created_by = self.created_by

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "identifier": identifier,
                "title": title,
                "text": text,
            }
        )
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if sentences is not UNSET:
            field_dict["sentences"] = sentences
        if categories is not UNSET:
            field_dict["categories"] = categories
        if annotations is not UNSET:
            field_dict["annotations"] = annotations
        if created_date is not UNSET:
            field_dict["createdDate"] = created_date
        if modified_date is not UNSET:
            field_dict["modifiedDate"] = modified_date
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        identifier = d.pop("identifier")

        title = d.pop("title")

        text = d.pop("text")

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, DocumentMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = DocumentMetadata.from_dict(_metadata)

        sentences = []
        _sentences = d.pop("sentences", UNSET)
        for sentences_item_data in _sentences or []:
            sentences_item = DocSentence.from_dict(sentences_item_data)

            sentences.append(sentences_item)

        categories = []
        _categories = d.pop("categories", UNSET)
        for categories_item_data in _categories or []:
            categories_item = DocCategory.from_dict(categories_item_data)

            categories.append(categories_item)

        annotations = []
        _annotations = d.pop("annotations", UNSET)
        for annotations_item_data in _annotations or []:
            annotations_item = DocAnnotation.from_dict(annotations_item_data)

            annotations.append(annotations_item)

        created_date = d.pop("createdDate", UNSET)

        modified_date = d.pop("modifiedDate", UNSET)

        created_by = d.pop("createdBy", UNSET)

        document = cls(
            identifier=identifier,
            title=title,
            text=text,
            metadata=metadata,
            sentences=sentences,
            categories=categories,
            annotations=annotations,
            created_date=created_date,
            modified_date=modified_date,
            created_by=created_by,
        )

        return document



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="Bucket")


@attr.s(auto_attribs=True)
class Bucket:
    """ """

    key: str
    doc_count: int

    def to_dict(self) -> Dict[str, Any]:
        key = self.key
        doc_count = self.doc_count

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "key": key,
                "doc_count": doc_count,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        key = d.pop("key")

        doc_count = d.pop("doc_count")

        bucket = cls(
            key=key,
            doc_count=doc_count,
        )

        return bucket



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="DocSentence")


@attr.s(auto_attribs=True)
class DocSentence:
    """ """

    start: int
    end: int

    def to_dict(self) -> Dict[str, Any]:
        start = self.start
        end = self.end

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "start": start,
                "end": end,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        start = d.pop("start")

        end = d.pop("end")

        doc_sentence = cls(
            start=start,
            end=end,
        )

        return doc_sentence



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Lexicon")


@attr.s(auto_attribs=True)
class Lexicon:
    """ """

    name: str
    label: str
    color: str
    manual_edition_allowed: bool
    terms: Union[Unset, int] = UNSET
    created_by: Union[Unset, str] = UNSET
    created_at: Union[Unset, str] = UNSET
    modified_by: Union[Unset, str] = UNSET
    modified_at: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        label = self.label
        color = self.color
        manual_edition_allowed = self.manual_edition_allowed
        terms = self.terms
        created_by = self.created_by
        created_at = self.created_at
        modified_by = self.modified_by
        modified_at = self.modified_at

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
                "label": label,
                "color": color,
                "manualEditionAllowed": manual_edition_allowed,
            }
        )
        if terms is not UNSET:
            field_dict["terms"] = terms
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if modified_by is not UNSET:
            field_dict["modifiedBy"] = modified_by
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        label = d.pop("label")

        color = d.pop("color")

        manual_edition_allowed = d.pop("manualEditionAllowed")

        terms = d.pop("terms", UNSET)

        created_by = d.pop("createdBy", UNSET)

        created_at = d.pop("createdAt", UNSET)

        modified_by = d.pop("modifiedBy", UNSET)

        modified_at = d.pop("modifiedAt", UNSET)

        lexicon = cls(
            name=name,
            label=label,
            color=color,
            manual_edition_allowed=manual_edition_allowed,
            terms=terms,
            created_by=created_by,
            created_at=created_at,
            modified_by=modified_by,
            modified_at=modified_at,
        )

        return lexicon



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.metadata_definition_entry import MetadataDefinitionEntry

T = TypeVar("T", bound="MetadataDefinition")


@attr.s(auto_attribs=True)
class MetadataDefinition:
    """ """

    metadata: List[MetadataDefinitionEntry]

    def to_dict(self) -> Dict[str, Any]:
        metadata = []
        for metadata_item_data in self.metadata:
            metadata_item = metadata_item_data.to_dict()

            metadata.append(metadata_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "metadata": metadata,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        metadata = []
        _metadata = d.pop("metadata")
        for metadata_item_data in _metadata:
            metadata_item = MetadataDefinitionEntry.from_dict(metadata_item_data)

            metadata.append(metadata_item)

        metadata_definition = cls(
            metadata=metadata,
        )

        return metadata_definition



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.sherpa_job_bean import SherpaJobBean
from ..types import UNSET, Unset

T = TypeVar("T", bound="ProjectStatus")


@attr.s(auto_attribs=True)
class ProjectStatus:
    """ """

    project_name: str
    status: str
    pending_job: Union[Unset, SherpaJobBean] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        project_name = self.project_name
        status = self.status
        pending_job: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pending_job, Unset):
            pending_job = self.pending_job.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "projectName": project_name,
                "status": status,
            }
        )
        if pending_job is not UNSET:
            field_dict["pendingJob"] = pending_job

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        project_name = d.pop("projectName")

        status = d.pop("status")

        _pending_job = d.pop("pendingJob", UNSET)
        pending_job: Union[Unset, SherpaJobBean]
        if isinstance(_pending_job, Unset):
            pending_job = UNSET
        else:
            pending_job = SherpaJobBean.from_dict(_pending_job)

        project_status = cls(
            project_name=project_name,
            status=status,
            pending_job=pending_job,
        )

        return project_status



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="UserPermissionsUpdate")


@attr.s(auto_attribs=True)
class UserPermissionsUpdate:
    """ """

    permissions: Union[Unset, List[str]] = UNSET
    roles: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        permissions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = self.permissions

        roles: Union[Unset, List[str]] = UNSET
        if not isinstance(self.roles, Unset):
            roles = self.roles

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if permissions is not UNSET:
            field_dict["permissions"] = permissions
        if roles is not UNSET:
            field_dict["roles"] = roles

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        permissions = cast(List[str], d.pop("permissions", UNSET))

        roles = cast(List[str], d.pop("roles", UNSET))

        user_permissions_update = cls(
            permissions=permissions,
            roles=roles,
        )

        return user_permissions_update



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="DeleteGroupResult")


@attr.s(auto_attribs=True)
class DeleteGroupResult:
    """ """

    removed_users: int
    removed_projects: int

    def to_dict(self) -> Dict[str, Any]:
        removed_users = self.removed_users
        removed_projects = self.removed_projects

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "removedUsers": removed_users,
                "removedProjects": removed_projects,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        removed_users = d.pop("removedUsers")

        removed_projects = d.pop("removedProjects")

        delete_group_result = cls(
            removed_users=removed_users,
            removed_projects=removed_projects,
        )

        return delete_group_result



from enum import Enum


class SherpaJobBeanType(str, Enum):
    DOC_IMPORT = "DOC_IMPORT"
    EXPERIMENT_TRAINING = "EXPERIMENT_TRAINING"
    DOC_DELETION = "DOC_DELETION"
    CORPUS_ANNOTATE = "CORPUS_ANNOTATE"
    TRAIN_AND_SUGGEST = "TRAIN_AND_SUGGEST"
    UNSPECIFIED = "UNSPECIFIED"
    PROJECT_INDEXING = "PROJECT_INDEXING"
    PROJECTS_INDEXING = "PROJECTS_INDEXING"
    PROJECT_RESTORATION = "PROJECT_RESTORATION"
    DELETE_LABELS = "DELETE_LABELS"
    PROCESS_CANDIDATES = "PROCESS_CANDIDATES"
    SYNCHRONIZE_TERMS = "SYNCHRONIZE_TERMS"
    TERM_IMPORT = "TERM_IMPORT"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.annotated_doc_annotation import AnnotatedDocAnnotation
from ..models.annotated_doc_category import AnnotatedDocCategory
from ..models.annotated_document_metadata import AnnotatedDocumentMetadata
from ..models.doc_sentence import DocSentence
from ..types import UNSET, Unset

T = TypeVar("T", bound="AnnotatedDocument")


@attr.s(auto_attribs=True)
class AnnotatedDocument:
    """ """

    text: str
    identifier: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    metadata: Union[Unset, AnnotatedDocumentMetadata] = UNSET
    sentences: Union[Unset, List[DocSentence]] = UNSET
    categories: Union[Unset, List[AnnotatedDocCategory]] = UNSET
    annotations: Union[Unset, List[AnnotatedDocAnnotation]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        text = self.text
        identifier = self.identifier
        title = self.title
        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        sentences: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.sentences, Unset):
            sentences = []
            for sentences_item_data in self.sentences:
                sentences_item = sentences_item_data.to_dict()

                sentences.append(sentences_item)

        categories: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.categories, Unset):
            categories = []
            for categories_item_data in self.categories:
                categories_item = categories_item_data.to_dict()

                categories.append(categories_item)

        annotations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.annotations, Unset):
            annotations = []
            for annotations_item_data in self.annotations:
                annotations_item = annotations_item_data.to_dict()

                annotations.append(annotations_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "text": text,
            }
        )
        if identifier is not UNSET:
            field_dict["identifier"] = identifier
        if title is not UNSET:
            field_dict["title"] = title
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if sentences is not UNSET:
            field_dict["sentences"] = sentences
        if categories is not UNSET:
            field_dict["categories"] = categories
        if annotations is not UNSET:
            field_dict["annotations"] = annotations

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        text = d.pop("text")

        identifier = d.pop("identifier", UNSET)

        title = d.pop("title", UNSET)

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, AnnotatedDocumentMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = AnnotatedDocumentMetadata.from_dict(_metadata)

        sentences = []
        _sentences = d.pop("sentences", UNSET)
        for sentences_item_data in _sentences or []:
            sentences_item = DocSentence.from_dict(sentences_item_data)

            sentences.append(sentences_item)

        categories = []
        _categories = d.pop("categories", UNSET)
        for categories_item_data in _categories or []:
            categories_item = AnnotatedDocCategory.from_dict(categories_item_data)

            categories.append(categories_item)

        annotations = []
        _annotations = d.pop("annotations", UNSET)
        for annotations_item_data in _annotations or []:
            annotations_item = AnnotatedDocAnnotation.from_dict(annotations_item_data)

            annotations.append(annotations_item)

        annotated_document = cls(
            text=text,
            identifier=identifier,
            title=title,
            metadata=metadata,
            sentences=sentences,
            categories=categories,
            annotations=annotations,
        )

        return annotated_document



from typing import Any, Dict, Type, TypeVar

import attr

from ..models.term_importer_spec_parameters import TermImporterSpecParameters

T = TypeVar("T", bound="TermImporterSpec")


@attr.s(auto_attribs=True)
class TermImporterSpec:
    """ """

    format_: str
    parameters: TermImporterSpecParameters

    def to_dict(self) -> Dict[str, Any]:
        format_ = self.format_
        parameters = self.parameters.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "format": format_,
                "parameters": parameters,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        format_ = d.pop("format")

        parameters = TermImporterSpecParameters.from_dict(d.pop("parameters"))

        term_importer_spec = cls(
            format_=format_,
            parameters=parameters,
        )

        return term_importer_spec



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="LabelCount")


@attr.s(auto_attribs=True)
class LabelCount:
    """ """

    id: str
    count: int

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        count = self.count

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "_id": id,
                "count": count,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("_id")

        count = d.pop("count")

        label_count = cls(
            id=id,
            count=count,
        )

        return label_count



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="OperationCount")


@attr.s(auto_attribs=True)
class OperationCount:
    """Annotation creation response"""

    operation: str
    unit: str
    count: int

    def to_dict(self) -> Dict[str, Any]:
        operation = self.operation
        unit = self.unit
        count = self.count

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "operation": operation,
                "unit": unit,
                "count": count,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        operation = d.pop("operation")

        unit = d.pop("unit")

        count = d.pop("count")

        operation_count = cls(
            operation=operation,
            unit=unit,
            count=count,
        )

        return operation_count



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.experiment_patch_parameters import ExperimentPatchParameters
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExperimentPatch")


@attr.s(auto_attribs=True)
class ExperimentPatch:
    """ """

    label: Union[Unset, str] = UNSET
    favorite: Union[Unset, bool] = UNSET
    parameters: Union[Unset, ExperimentPatchParameters] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        label = self.label
        favorite = self.favorite
        parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parameters, Unset):
            parameters = self.parameters.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if label is not UNSET:
            field_dict["label"] = label
        if favorite is not UNSET:
            field_dict["favorite"] = favorite
        if parameters is not UNSET:
            field_dict["parameters"] = parameters

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        label = d.pop("label", UNSET)

        favorite = d.pop("favorite", UNSET)

        _parameters = d.pop("parameters", UNSET)
        parameters: Union[Unset, ExperimentPatchParameters]
        if isinstance(_parameters, Unset):
            parameters = UNSET
        else:
            parameters = ExperimentPatchParameters.from_dict(_parameters)

        experiment_patch = cls(
            label=label,
            favorite=favorite,
            parameters=parameters,
        )

        return experiment_patch



""" Contains all the data models used in inputs/outputs """

from .ack import Ack
from .aggregation import Aggregation
from .annotate_binary_form import AnnotateBinaryForm
from .annotate_binary_with_plan_ref_multipart_data import AnnotateBinaryWithPlanRefMultipartData
from .annotate_documents_with_many import AnnotateDocumentsWithMany
from .annotate_format_binary_with_plan_ref_multipart_data import AnnotateFormatBinaryWithPlanRefMultipartData
from .annotate_text_with_many import AnnotateTextWithMany
from .annotated_doc_annotation import AnnotatedDocAnnotation
from .annotated_doc_annotation_properties import AnnotatedDocAnnotationProperties
from .annotated_doc_category import AnnotatedDocCategory
from .annotated_doc_category_properties import AnnotatedDocCategoryProperties
from .annotated_document import AnnotatedDocument
from .annotated_document_metadata import AnnotatedDocumentMetadata
from .annotation import Annotation
from .annotation_facets import AnnotationFacets
from .annotation_id import AnnotationId
from .annotation_metrics import AnnotationMetrics
from .annotation_plan import AnnotationPlan
from .annotation_status import AnnotationStatus
from .annotation_term import AnnotationTerm
from .annotation_term_properties import AnnotationTermProperties
from .annotator import Annotator
from .annotator_multimap import AnnotatorMultimap
from .bearer_token import BearerToken
from .bucket import Bucket
from .categories_facets import CategoriesFacets
from .category import Category
from .category_id import CategoryId
from .category_metrics import CategoryMetrics
from .category_status import CategoryStatus
from .classification_config import ClassificationConfig
from .classification_options import ClassificationOptions
from .config_patch_options import ConfigPatchOptions
from .convert_annotation_plan import ConvertAnnotationPlan
from .convert_format_annotation_plan import ConvertFormatAnnotationPlan
from .converter import Converter
from .converter_parameters import ConverterParameters
from .corpus_metrics import CorpusMetrics
from .create_lexicon_response_200 import CreateLexiconResponse200
from .create_project_from_archive_multipart_data import CreateProjectFromArchiveMultipartData
from .create_term_response_200 import CreateTermResponse200
from .created_by_count import CreatedByCount
from .credentials import Credentials
from .delete_group_result import DeleteGroupResult
from .delete_response import DeleteResponse
from .doc_annotation import DocAnnotation
from .doc_annotation_status import DocAnnotationStatus
from .doc_category import DocCategory
from .doc_category_status import DocCategoryStatus
from .doc_sentence import DocSentence
from .document import Document
from .document_facets import DocumentFacets
from .document_hit import DocumentHit
from .document_hits import DocumentHits
from .document_metadata import DocumentMetadata
from .engine_config import EngineConfig
from .engine_config_import_summary import EngineConfigImportSummary
from .engine_name import EngineName
from .experiment import Experiment
from .experiment_parameters import ExperimentParameters
from .experiment_patch import ExperimentPatch
from .experiment_patch_parameters import ExperimentPatchParameters
from .format_binary_form import FormatBinaryForm
from .format_documents_with_many import FormatDocumentsWithMany
from .format_text_with_many import FormatTextWithMany
from .formatter import Formatter
from .formatter_parameters import FormatterParameters
from .gazetteer import Gazetteer
from .gazetteer_parameters import GazetteerParameters
from .gazetteer_patch import GazetteerPatch
from .gazetteer_patch_parameters import GazetteerPatchParameters
from .generated_label_hint import GeneratedLabelHint
from .get_app_state_response_200 import GetAppStateResponse200
from .get_engine_parameters_schema_response_200 import GetEngineParametersSchemaResponse200
from .get_project_engine_parameters_schema_response_200 import GetProjectEngineParametersSchemaResponse200
from .get_segment_context_context_output import GetSegmentContextContextOutput
from .get_suggestions_response_200 import GetSuggestionsResponse200
from .group_desc import GroupDesc
from .group_name import GroupName
from .group_patch import GroupPatch
from .group_share import GroupShare
from .http_service_metadata import HttpServiceMetadata
from .http_service_metadata_operations import HttpServiceMetadataOperations
from .http_service_record import HttpServiceRecord
from .import_archive_multipart_data import ImportArchiveMultipartData
from .import_models_multipart_data import ImportModelsMultipartData
from .input_document import InputDocument
from .input_document_metadata import InputDocumentMetadata
from .input_label import InputLabel
from .job_status import JobStatus
from .job_type import JobType
from .label import Label
from .label_count import LabelCount
from .label_update import LabelUpdate
from .launch_document_import_multipart_data import LaunchDocumentImportMultipartData
from .launch_document_import_segmentation_policy import LaunchDocumentImportSegmentationPolicy
from .launch_project_restoration_from_backup_multipart_data import LaunchProjectRestorationFromBackupMultipartData
from .launch_uploaded_document_import_segmentation_policy import LaunchUploadedDocumentImportSegmentationPolicy
from .lexicon import Lexicon
from .lexicon_update import LexiconUpdate
from .maybe_create_projects_and_import_models_from_archive_multipart_data import (
    MaybeCreateProjectsAndImportModelsFromArchiveMultipartData,
)
from .metadata_count import MetadataCount
from .metadata_definition import MetadataDefinition
from .metadata_definition_entry import MetadataDefinitionEntry
from .model_metrics import ModelMetrics
from .model_metrics_options import ModelMetricsOptions
from .models_metrics import ModelsMetrics
from .named_annotation_plan import NamedAnnotationPlan
from .new_experiment import NewExperiment
from .new_experiment_parameters import NewExperimentParameters
from .new_gazetteer import NewGazetteer
from .new_gazetteer_parameters import NewGazetteerParameters
from .new_group_desc import NewGroupDesc
from .new_named_annotation_plan import NewNamedAnnotationPlan
from .new_role import NewRole
from .new_suggester import NewSuggester
from .new_suggester_parameters import NewSuggesterParameters
from .new_user import NewUser
from .operation_count import OperationCount
from .partial_label import PartialLabel
from .partial_lexicon import PartialLexicon
from .plan_patch import PlanPatch
from .project_bean import ProjectBean
from .project_config_creation import ProjectConfigCreation
from .project_config_creation_properties import ProjectConfigCreationProperties
from .project_property import ProjectProperty
from .project_status import ProjectStatus
from .quality_figures import QualityFigures
from .relation import Relation
from .report import Report
from .report_classes import ReportClasses
from .request_jwt_token_project_access_mode import RequestJwtTokenProjectAccessMode
from .role_desc import RoleDesc
from .role_update import RoleUpdate
from .search_total import SearchTotal
from .search_total_relation import SearchTotalRelation
from .segment import Segment
from .segment_context import SegmentContext
from .segment_contexts import SegmentContexts
from .segment_hit import SegmentHit
from .segment_hits import SegmentHits
from .segment_metadata import SegmentMetadata
from .share_mode import ShareMode
from .sherpa_job_bean import SherpaJobBean
from .sherpa_job_bean_status import SherpaJobBeanStatus
from .sherpa_job_bean_type import SherpaJobBeanType
from .simple_metadata import SimpleMetadata
from .suggester import Suggester
from .suggester_parameters import SuggesterParameters
from .suggester_patch import SuggesterPatch
from .suggester_patch_parameters import SuggesterPatchParameters
from .suggestion_facets import SuggestionFacets
from .term import Term
from .term_hit import TermHit
from .term_hit_term import TermHitTerm
from .term_hits import TermHits
from .term_import import TermImport
from .term_importer_spec import TermImporterSpec
from .term_importer_spec_parameters import TermImporterSpecParameters
from .text_count import TextCount
from .upload_files_multipart_data import UploadFilesMultipartData
from .uploaded_file import UploadedFile
from .uploaded_file_info import UploadedFileInfo
from .user_permissions_update import UserPermissionsUpdate
from .user_profile_update import UserProfileUpdate
from .user_response import UserResponse
from .with_annotator import WithAnnotator
from .with_annotator_condition import WithAnnotatorCondition
from .with_annotator_parameters import WithAnnotatorParameters
from .with_processor import WithProcessor
from .with_processor_condition import WithProcessorCondition
from .with_processor_parameters import WithProcessorParameters



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.converter import Converter
from ..models.formatter import Formatter
from ..models.with_annotator import WithAnnotator
from ..models.with_processor import WithProcessor
from ..types import UNSET, Unset

T = TypeVar("T", bound="AnnotationPlan")


@attr.s(auto_attribs=True)
class AnnotationPlan:
    """ """

    pipeline: List[Union[WithAnnotator, WithProcessor]]
    converter: Union[Unset, Converter] = UNSET
    formatter: Union[Unset, Formatter] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        pipeline = []
        for pipeline_item_data in self.pipeline:
            if isinstance(pipeline_item_data, WithAnnotator):
                pipeline_item = pipeline_item_data.to_dict()

            else:
                pipeline_item = pipeline_item_data.to_dict()

            pipeline.append(pipeline_item)

        converter: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.converter, Unset):
            converter = self.converter.to_dict()

        formatter: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.formatter, Unset):
            formatter = self.formatter.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "pipeline": pipeline,
            }
        )
        if converter is not UNSET:
            field_dict["converter"] = converter
        if formatter is not UNSET:
            field_dict["formatter"] = formatter

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        pipeline = []
        _pipeline = d.pop("pipeline")
        for pipeline_item_data in _pipeline:

            def _parse_pipeline_item(data: object) -> Union[WithAnnotator, WithProcessor]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    pipeline_item_type_0 = WithAnnotator.from_dict(data)

                    return pipeline_item_type_0
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                pipeline_item_type_1 = WithProcessor.from_dict(data)

                return pipeline_item_type_1

            pipeline_item = _parse_pipeline_item(pipeline_item_data)

            pipeline.append(pipeline_item)

        _converter = d.pop("converter", UNSET)
        converter: Union[Unset, Converter]
        if isinstance(_converter, Unset):
            converter = UNSET
        else:
            converter = Converter.from_dict(_converter)

        _formatter = d.pop("formatter", UNSET)
        formatter: Union[Unset, Formatter]
        if isinstance(_formatter, Unset):
            formatter = UNSET
        else:
            formatter = Formatter.from_dict(_formatter)

        annotation_plan = cls(
            pipeline=pipeline,
            converter=converter,
            formatter=formatter,
        )

        return annotation_plan



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.bearer_token import BearerToken
from ...models.credentials import Credentials
from ...models.request_jwt_token_project_access_mode import RequestJwtTokenProjectAccessMode
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    json_body: Credentials,
    project_filter: Union[Unset, None, str] = UNSET,
    project_access_mode: Union[Unset, None, RequestJwtTokenProjectAccessMode] = UNSET,
    annotate_only: Union[Unset, None, bool] = False,
    login_only: Union[Unset, None, bool] = False,
) -> Dict[str, Any]:
    url = "{}/auth/login".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_project_access_mode: Union[Unset, None, str] = UNSET
    if not isinstance(project_access_mode, Unset):
        json_project_access_mode = project_access_mode.value if project_access_mode else None

    params: Dict[str, Any] = {
        "projectFilter": project_filter,
        "projectAccessMode": json_project_access_mode,
        "annotateOnly": annotate_only,
        "loginOnly": login_only,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[BearerToken]:
    if response.status_code == 200:
        response_200 = BearerToken.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[BearerToken]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    json_body: Credentials,
    project_filter: Union[Unset, None, str] = UNSET,
    project_access_mode: Union[Unset, None, RequestJwtTokenProjectAccessMode] = UNSET,
    annotate_only: Union[Unset, None, bool] = False,
    login_only: Union[Unset, None, bool] = False,
) -> Response[BearerToken]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
        project_filter=project_filter,
        project_access_mode=project_access_mode,
        annotate_only=annotate_only,
        login_only=login_only,
    )

    response = httpx.post(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    json_body: Credentials,
    project_filter: Union[Unset, None, str] = UNSET,
    project_access_mode: Union[Unset, None, RequestJwtTokenProjectAccessMode] = UNSET,
    annotate_only: Union[Unset, None, bool] = False,
    login_only: Union[Unset, None, bool] = False,
) -> Optional[BearerToken]:
    """ """

    return sync_detailed(
        client=client,
        json_body=json_body,
        project_filter=project_filter,
        project_access_mode=project_access_mode,
        annotate_only=annotate_only,
        login_only=login_only,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    json_body: Credentials,
    project_filter: Union[Unset, None, str] = UNSET,
    project_access_mode: Union[Unset, None, RequestJwtTokenProjectAccessMode] = UNSET,
    annotate_only: Union[Unset, None, bool] = False,
    login_only: Union[Unset, None, bool] = False,
) -> Response[BearerToken]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
        project_filter=project_filter,
        project_access_mode=project_access_mode,
        annotate_only=annotate_only,
        login_only=login_only,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    json_body: Credentials,
    project_filter: Union[Unset, None, str] = UNSET,
    project_access_mode: Union[Unset, None, RequestJwtTokenProjectAccessMode] = UNSET,
    annotate_only: Union[Unset, None, bool] = False,
    login_only: Union[Unset, None, bool] = False,
) -> Optional[BearerToken]:
    """ """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
            project_filter=project_filter,
            project_access_mode=project_access_mode,
            annotate_only=annotate_only,
            login_only=login_only,
        )
    ).parsed



from enum import Enum


class GetSegmentContextContextOutput(str, Enum):
    SEGMENTS = "segments"
    MERGED_SEGMENTS = "merged_segments"
    ALL = "all"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="Relation")


@attr.s(auto_attribs=True)
class Relation:
    """ """

    def to_dict(self) -> Dict[str, Any]:

        field_dict: Dict[str, Any] = {}
        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        src_dict.copy()
        relation = cls()

        return relation



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="LabelUpdate")


@attr.s(auto_attribs=True)
class LabelUpdate:
    """ """

    label: Union[Unset, str] = UNSET
    color: Union[Unset, str] = UNSET
    identifier: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        label = self.label
        color = self.color
        identifier = self.identifier

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if label is not UNSET:
            field_dict["label"] = label
        if color is not UNSET:
            field_dict["color"] = color
        if identifier is not UNSET:
            field_dict["identifier"] = identifier

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        label = d.pop("label", UNSET)

        color = d.pop("color", UNSET)

        identifier = d.pop("identifier", UNSET)

        label_update = cls(
            label=label,
            color=color,
            identifier=identifier,
        )

        return label_update



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sherpa_client',
 'sherpa_client.api',
 'sherpa_client.api.annotate',
 'sherpa_client.api.annotations',
 'sherpa_client.api.annotators',
 'sherpa_client.api.authentication',
 'sherpa_client.api.categories',
 'sherpa_client.api.documents',
 'sherpa_client.api.experiments',
 'sherpa_client.api.gazetteers',
 'sherpa_client.api.groups',
 'sherpa_client.api.jobs',
 'sherpa_client.api.labels',
 'sherpa_client.api.lexicons',
 'sherpa_client.api.metrics',
 'sherpa_client.api.models',
 'sherpa_client.api.plans',
 'sherpa_client.api.project_engines',
 'sherpa_client.api.projects',
 'sherpa_client.api.roles',
 'sherpa_client.api.segments',
 'sherpa_client.api.services',
 'sherpa_client.api.shares',
 'sherpa_client.api.suggesters',
 'sherpa_client.api.suggestions',
 'sherpa_client.api.uploads',
 'sherpa_client.api.users',
 'sherpa_client.models']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=20.1.0,<22.0.0',
 'httpx>=0.15.4,<0.21.0',
 'python-dateutil>=2.8.0,<3.0.0']

setup_kwargs = {
    'name': 'sherpa-client',
    'version': '0.9.1',
    'description': 'A client library for accessing Sherpa API documentation (Iota release)',
    'long_description': '# sherpa-client\nA client library for accessing Sherpa API documentation\n\n## Usage\nFirst, create a client:\n\n```python\nfrom sherpa_client import Client\n\nclient = Client(base_url="https://api.example.com")\n```\n\nIf the endpoints you\'re going to hit require authentication, use `AuthenticatedClient` instead:\n\n```python\nfrom sherpa_client import AuthenticatedClient\n\nclient = AuthenticatedClient(base_url="https://api.example.com", token="SuperSecretToken")\n```\n\nNow call your endpoint and use your models:\n\n```python\nfrom sherpa_client.models import MyDataModel\nfrom sherpa_client.api.my_tag import get_my_data_model\nfrom sherpa_client.types import Response\n\nmy_data: MyDataModel = get_my_data_model.sync(client=client)\n# or if you need more info (e.g. status_code)\nresponse: Response[MyDataModel] = get_my_data_model.sync_detailed(client=client)\n```\n\nOr do the same thing with an async version:\n\n```python\nfrom sherpa_client.models import MyDataModel\nfrom sherpa_client.api.my_tag import get_my_data_model\nfrom sherpa_client.types import Response\n\nmy_data: MyDataModel = await get_my_data_model.asyncio(client=client)\nresponse: Response[MyDataModel] = await get_my_data_model.asyncio_detailed(client=client)\n```\n\nBy default, when you\'re calling an HTTPS API it will attempt to verify that SSL is working correctly. Using certificate verification is highly recommended most of the time, but sometimes you may need to authenticate to a server (especially an internal server) using a custom certificate bundle.\n\n```python\nclient = AuthenticatedClient(\n    base_url="https://internal_api.example.com", \n    token="SuperSecretToken",\n    verify_ssl="/path/to/certificate_bundle.pem",\n)\n```\n\nYou can also disable certificate validation altogether, but beware that **this is a security risk**.\n\n```python\nclient = AuthenticatedClient(\n    base_url="https://internal_api.example.com", \n    token="SuperSecretToken", \n    verify_ssl=False\n)\n```\n\nThings to know:\n1. Every path/method combo becomes a Python module with four functions:\n    1. `sync`: Blocking request that returns parsed data (if successful) or `None`\n    1. `sync_detailed`: Blocking request that always returns a `Request`, optionally with `parsed` set if the request was successful.\n    1. `asyncio`: Like `sync` but the async instead of blocking\n    1. `asyncio_detailed`: Like `sync_detailed` by async instead of blocking\n\n1. All path/query params, and bodies become method arguments.\n1. If your endpoint had any tags on it, the first tag will be used as a module name for the function (my_tag above)\n1. Any endpoint which did not have a tag will be in `sherpa_client.api.default`\n\n## Building / publishing this Client\nThis project uses [Poetry](https://python-poetry.org/) to manage dependencies  and packaging.  Here are the basics:\n1. Update the metadata in pyproject.toml (e.g. authors, version)\n1. If you\'re using a private repository, configure it with Poetry\n    1. `poetry config repositories.<your-repository-name> <url-to-your-repository>`\n    1. `poetry config http-basic.<your-repository-name> <username> <password>`\n1. Publish the client with `poetry publish --build -r <your-repository-name>` or, if for public PyPI, just `poetry publish --build`\n\nIf you want to install this client into another project without publishing it (e.g. for development) then:\n1. If that project **is using Poetry**, you can simply do `poetry add <path-to-this-client>` from that project\n1. If that project is not using Poetry:\n    1. Build a wheel with `poetry build -f wheel`\n    1. Install that wheel from the other project `pip install <path-to-wheel>`',
    'author': None,
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import Any, Dict, Type, TypeVar

import attr

from ..models.new_experiment_parameters import NewExperimentParameters

T = TypeVar("T", bound="NewExperiment")


@attr.s(auto_attribs=True)
class NewExperiment:
    """ """

    engine: str
    label: str
    parameters: NewExperimentParameters

    def to_dict(self) -> Dict[str, Any]:
        engine = self.engine
        label = self.label
        parameters = self.parameters.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "engine": engine,
                "label": label,
                "parameters": parameters,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        engine = d.pop("engine")

        label = d.pop("label")

        parameters = NewExperimentParameters.from_dict(d.pop("parameters"))

        new_experiment = cls(
            engine=engine,
            label=label,
            parameters=parameters,
        )

        return new_experiment



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.annotation_term_properties import AnnotationTermProperties
from ..types import UNSET, Unset

T = TypeVar("T", bound="AnnotationTerm")


@attr.s(auto_attribs=True)
class AnnotationTerm:
    """A term"""

    identifier: str
    lexicon: str
    preferred_form: Union[Unset, str] = UNSET
    score: Union[Unset, int] = UNSET
    properties: Union[Unset, AnnotationTermProperties] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        identifier = self.identifier
        lexicon = self.lexicon
        preferred_form = self.preferred_form
        score = self.score
        properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.properties, Unset):
            properties = self.properties.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "identifier": identifier,
                "lexicon": lexicon,
            }
        )
        if preferred_form is not UNSET:
            field_dict["preferredForm"] = preferred_form
        if score is not UNSET:
            field_dict["score"] = score
        if properties is not UNSET:
            field_dict["properties"] = properties

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        identifier = d.pop("identifier")

        lexicon = d.pop("lexicon")

        preferred_form = d.pop("preferredForm", UNSET)

        score = d.pop("score", UNSET)

        _properties = d.pop("properties", UNSET)
        properties: Union[Unset, AnnotationTermProperties]
        if isinstance(_properties, Unset):
            properties = UNSET
        else:
            properties = AnnotationTermProperties.from_dict(_properties)

        annotation_term = cls(
            identifier=identifier,
            lexicon=lexicon,
            preferred_form=preferred_form,
            score=score,
            properties=properties,
        )

        return annotation_term



from typing import Any, Dict, List, Type, TypeVar, cast

import attr

T = TypeVar("T", bound="NewUser")


@attr.s(auto_attribs=True)
class NewUser:
    """ """

    username: str
    password: str
    permissions: List[str]
    roles: List[str]

    def to_dict(self) -> Dict[str, Any]:
        username = self.username
        password = self.password
        permissions = self.permissions

        roles = self.roles

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "username": username,
                "password": password,
                "permissions": permissions,
                "roles": roles,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        username = d.pop("username")

        password = d.pop("password")

        permissions = cast(List[str], d.pop("permissions"))

        roles = cast(List[str], d.pop("roles"))

        new_user = cls(
            username=username,
            password=password,
            permissions=permissions,
            roles=roles,
        )

        return new_user



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ClassificationOptions")


@attr.s(auto_attribs=True)
class ClassificationOptions:
    """ """

    exclusive_classes: Union[Unset, bool] = True

    def to_dict(self) -> Dict[str, Any]:
        exclusive_classes = self.exclusive_classes

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if exclusive_classes is not UNSET:
            field_dict["exclusive_classes"] = exclusive_classes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        exclusive_classes = d.pop("exclusive_classes", UNSET)

        classification_options = cls(
            exclusive_classes=exclusive_classes,
        )

        return classification_options



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.doc_annotation_status import DocAnnotationStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="DocAnnotation")


@attr.s(auto_attribs=True)
class DocAnnotation:
    """A document annotation"""

    label_name: str
    start: int
    end: int
    text: str
    identifier: Union[Unset, str] = UNSET
    status: Union[Unset, DocAnnotationStatus] = UNSET
    created_date: Union[Unset, str] = UNSET
    modified_date: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        label_name = self.label_name
        start = self.start
        end = self.end
        text = self.text
        identifier = self.identifier
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        created_date = self.created_date
        modified_date = self.modified_date
        created_by = self.created_by

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "labelName": label_name,
                "start": start,
                "end": end,
                "text": text,
            }
        )
        if identifier is not UNSET:
            field_dict["identifier"] = identifier
        if status is not UNSET:
            field_dict["status"] = status
        if created_date is not UNSET:
            field_dict["createdDate"] = created_date
        if modified_date is not UNSET:
            field_dict["modifiedDate"] = modified_date
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        label_name = d.pop("labelName")

        start = d.pop("start")

        end = d.pop("end")

        text = d.pop("text")

        identifier = d.pop("identifier", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, DocAnnotationStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = DocAnnotationStatus(_status)

        created_date = d.pop("createdDate", UNSET)

        modified_date = d.pop("modifiedDate", UNSET)

        created_by = d.pop("createdBy", UNSET)

        doc_annotation = cls(
            label_name=label_name,
            start=start,
            end=end,
            text=text,
            identifier=identifier,
            status=status,
            created_date=created_date,
            modified_date=modified_date,
            created_by=created_by,
        )

        return doc_annotation



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.annotation_status import AnnotationStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="Annotation")


@attr.s(auto_attribs=True)
class Annotation:
    """A document annotation"""

    document_identifier: str
    label_name: str
    start: int
    end: int
    text: str
    status: Union[Unset, AnnotationStatus] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        document_identifier = self.document_identifier
        label_name = self.label_name
        start = self.start
        end = self.end
        text = self.text
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "documentIdentifier": document_identifier,
                "labelName": label_name,
                "start": start,
                "end": end,
                "text": text,
            }
        )
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        document_identifier = d.pop("documentIdentifier")

        label_name = d.pop("labelName")

        start = d.pop("start")

        end = d.pop("end")

        text = d.pop("text")

        _status = d.pop("status", UNSET)
        status: Union[Unset, AnnotationStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = AnnotationStatus(_status)

        annotation = cls(
            document_identifier=document_identifier,
            label_name=label_name,
            start=start,
            end=end,
            text=text,
            status=status,
        )

        return annotation



import json
from io import BytesIO
from typing import Any, Dict, Tuple, Type, TypeVar, Union

import attr

from ..models.convert_format_annotation_plan import ConvertFormatAnnotationPlan
from ..types import UNSET, File, FileJsonType, Unset

T = TypeVar("T", bound="FormatBinaryForm")


@attr.s(auto_attribs=True)
class FormatBinaryForm:
    """ """

    file: Union[Unset, File] = UNSET
    plan: Union[Unset, ConvertFormatAnnotationPlan] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        file: Union[Unset, FileJsonType] = UNSET
        if not isinstance(self.file, Unset):
            file = self.file.to_tuple()

        plan: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.plan, Unset):
            plan = self.plan.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if file is not UNSET:
            field_dict["file"] = file
        if plan is not UNSET:
            field_dict["plan"] = plan

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        file: Union[Unset, FileJsonType] = UNSET
        if not isinstance(self.file, Unset):
            file = self.file.to_tuple()

        plan: Union[Unset, Tuple[None, str, str]] = UNSET
        if not isinstance(self.plan, Unset):
            plan = (None, json.dumps(self.plan.to_dict()), "application/json")

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if file is not UNSET:
            field_dict["file"] = file
        if plan is not UNSET:
            field_dict["plan"] = plan

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _file = d.pop("file", UNSET)
        file: Union[Unset, File]
        if isinstance(_file, Unset):
            file = UNSET
        else:
            file = File(payload=BytesIO(_file))

        _plan = d.pop("plan", UNSET)
        plan: Union[Unset, ConvertFormatAnnotationPlan]
        if isinstance(_plan, Unset):
            plan = UNSET
        else:
            plan = ConvertFormatAnnotationPlan.from_dict(_plan)

        format_binary_form = cls(
            file=file,
            plan=plan,
        )

        return format_binary_form



from enum import Enum


class LaunchUploadedDocumentImportSegmentationPolicy(str, Enum):
    ALWAYS_RECOMPUTE = "always_recompute"
    COMPUTE_IF_MISSING = "compute_if_missing"
    NO_SEGMENTATION = "no_segmentation"
    DOCUMENTS_AS_SEGMENTS = "documents_as_segments"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

import attr

from ..models.segment_context import SegmentContext

T = TypeVar("T", bound="SegmentContexts")


@attr.s(auto_attribs=True)
class SegmentContexts:
    """ """

    before: SegmentContext
    after: SegmentContext

    def to_dict(self) -> Dict[str, Any]:
        before = self.before.to_dict()

        after = self.after.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "before": before,
                "after": after,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        before = SegmentContext.from_dict(d.pop("before"))

        after = SegmentContext.from_dict(d.pop("after"))

        segment_contexts = cls(
            before=before,
            after=after,
        )

        return segment_contexts



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.create_project_from_archive_multipart_data import CreateProjectFromArchiveMultipartData
from ...models.sherpa_job_bean import SherpaJobBean
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    multipart_data: CreateProjectFromArchiveMultipartData,
    group_name: Union[Unset, None, str] = UNSET,
    reuse_project_name: Union[Unset, None, bool] = False,
    project_name: Union[Unset, None, str] = UNSET,
    project_label: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/projects/_import".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {
        "groupName": group_name,
        "reuseProjectName": reuse_project_name,
        "projectName": project_name,
        "projectLabel": project_label,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    multipart_multipart_data = multipart_data.to_multipart()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "files": multipart_multipart_data,
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[SherpaJobBean]:
    if response.status_code == 200:
        response_200 = SherpaJobBean.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[SherpaJobBean]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    multipart_data: CreateProjectFromArchiveMultipartData,
    group_name: Union[Unset, None, str] = UNSET,
    reuse_project_name: Union[Unset, None, bool] = False,
    project_name: Union[Unset, None, str] = UNSET,
    project_label: Union[Unset, None, str] = UNSET,
) -> Response[SherpaJobBean]:
    kwargs = _get_kwargs(
        client=client,
        multipart_data=multipart_data,
        group_name=group_name,
        reuse_project_name=reuse_project_name,
        project_name=project_name,
        project_label=project_label,
    )

    response = httpx.post(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    multipart_data: CreateProjectFromArchiveMultipartData,
    group_name: Union[Unset, None, str] = UNSET,
    reuse_project_name: Union[Unset, None, bool] = False,
    project_name: Union[Unset, None, str] = UNSET,
    project_label: Union[Unset, None, str] = UNSET,
) -> Optional[SherpaJobBean]:
    """ """

    return sync_detailed(
        client=client,
        multipart_data=multipart_data,
        group_name=group_name,
        reuse_project_name=reuse_project_name,
        project_name=project_name,
        project_label=project_label,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    multipart_data: CreateProjectFromArchiveMultipartData,
    group_name: Union[Unset, None, str] = UNSET,
    reuse_project_name: Union[Unset, None, bool] = False,
    project_name: Union[Unset, None, str] = UNSET,
    project_label: Union[Unset, None, str] = UNSET,
) -> Response[SherpaJobBean]:
    kwargs = _get_kwargs(
        client=client,
        multipart_data=multipart_data,
        group_name=group_name,
        reuse_project_name=reuse_project_name,
        project_name=project_name,
        project_label=project_label,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    multipart_data: CreateProjectFromArchiveMultipartData,
    group_name: Union[Unset, None, str] = UNSET,
    reuse_project_name: Union[Unset, None, bool] = False,
    project_name: Union[Unset, None, str] = UNSET,
    project_label: Union[Unset, None, str] = UNSET,
) -> Optional[SherpaJobBean]:
    """ """

    return (
        await asyncio_detailed(
            client=client,
            multipart_data=multipart_data,
            group_name=group_name,
            reuse_project_name=reuse_project_name,
            project_name=project_name,
            project_label=project_label,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar

import attr

from ..models.term_hit_term import TermHitTerm

T = TypeVar("T", bound="TermHit")


@attr.s(auto_attribs=True)
class TermHit:
    """ """

    score: float
    term: TermHitTerm

    def to_dict(self) -> Dict[str, Any]:
        score = self.score
        term = self.term.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "score": score,
                "term": term,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        score = d.pop("score")

        term = TermHitTerm.from_dict(d.pop("term"))

        term_hit = cls(
            score=score,
            term=term,
        )

        return term_hit



from typing import Any, Dict, Type, TypeVar

import attr

from ..models.suggester_parameters import SuggesterParameters

T = TypeVar("T", bound="Suggester")


@attr.s(auto_attribs=True)
class Suggester:
    """ """

    name: str
    label: str
    engine: str
    running: bool
    quality: int
    timestamp: int
    duration: int
    uptodate: bool
    models: int
    parameters: SuggesterParameters

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        label = self.label
        engine = self.engine
        running = self.running
        quality = self.quality
        timestamp = self.timestamp
        duration = self.duration
        uptodate = self.uptodate
        models = self.models
        parameters = self.parameters.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
                "label": label,
                "engine": engine,
                "running": running,
                "quality": quality,
                "timestamp": timestamp,
                "duration": duration,
                "uptodate": uptodate,
                "models": models,
                "parameters": parameters,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        label = d.pop("label")

        engine = d.pop("engine")

        running = d.pop("running")

        quality = d.pop("quality")

        timestamp = d.pop("timestamp")

        duration = d.pop("duration")

        uptodate = d.pop("uptodate")

        models = d.pop("models")

        parameters = SuggesterParameters.from_dict(d.pop("parameters"))

        suggester = cls(
            name=name,
            label=label,
            engine=engine,
            running=running,
            quality=quality,
            timestamp=timestamp,
            duration=duration,
            uptodate=uptodate,
            models=models,
            parameters=parameters,
        )

        return suggester



from typing import Any, Dict, List, Optional

import httpx

from ...client import Client
from ...models.created_by_count import CreatedByCount
from ...types import Response


def _get_kwargs(
    project_name: str,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/projects/{projectName}/categories/_count_creators".format(client.base_url, projectName=project_name)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[List[CreatedByCount]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for componentsschemas_created_by_count_array_item_data in _response_200:
            componentsschemas_created_by_count_array_item = CreatedByCount.from_dict(
                componentsschemas_created_by_count_array_item_data
            )

            response_200.append(componentsschemas_created_by_count_array_item)

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[List[CreatedByCount]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    project_name: str,
    *,
    client: Client,
) -> Response[List[CreatedByCount]]:
    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
    )

    response = httpx.post(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    project_name: str,
    *,
    client: Client,
) -> Optional[List[CreatedByCount]]:
    """ """

    return sync_detailed(
        project_name=project_name,
        client=client,
    ).parsed


async def asyncio_detailed(
    project_name: str,
    *,
    client: Client,
) -> Response[List[CreatedByCount]]:
    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    project_name: str,
    *,
    client: Client,
) -> Optional[List[CreatedByCount]]:
    """ """

    return (
        await asyncio_detailed(
            project_name=project_name,
            client=client,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.formatter import Formatter
from ..models.input_document import InputDocument
from ..models.with_annotator import WithAnnotator
from ..models.with_processor import WithProcessor

T = TypeVar("T", bound="FormatDocumentsWithMany")


@attr.s(auto_attribs=True)
class FormatDocumentsWithMany:
    """ """

    documents: List[InputDocument]
    pipeline: List[Union[WithAnnotator, WithProcessor]]
    formatter: Formatter

    def to_dict(self) -> Dict[str, Any]:
        documents = []
        for documents_item_data in self.documents:
            documents_item = documents_item_data.to_dict()

            documents.append(documents_item)

        pipeline = []
        for pipeline_item_data in self.pipeline:
            if isinstance(pipeline_item_data, WithAnnotator):
                pipeline_item = pipeline_item_data.to_dict()

            else:
                pipeline_item = pipeline_item_data.to_dict()

            pipeline.append(pipeline_item)

        formatter = self.formatter.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "documents": documents,
                "pipeline": pipeline,
                "formatter": formatter,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        documents = []
        _documents = d.pop("documents")
        for documents_item_data in _documents:
            documents_item = InputDocument.from_dict(documents_item_data)

            documents.append(documents_item)

        pipeline = []
        _pipeline = d.pop("pipeline")
        for pipeline_item_data in _pipeline:

            def _parse_pipeline_item(data: object) -> Union[WithAnnotator, WithProcessor]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    pipeline_item_type_0 = WithAnnotator.from_dict(data)

                    return pipeline_item_type_0
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                pipeline_item_type_1 = WithProcessor.from_dict(data)

                return pipeline_item_type_1

            pipeline_item = _parse_pipeline_item(pipeline_item_data)

            pipeline.append(pipeline_item)

        formatter = Formatter.from_dict(d.pop("formatter"))

        format_documents_with_many = cls(
            documents=documents,
            pipeline=pipeline,
            formatter=formatter,
        )

        return format_documents_with_many
